searchNodes=[{"doc":"JViewer is an excellent way to declaratively represent elixir data in a JSON encodable format. Imagine you have an application that stores data in a relational database. When your application grows large, data becomes to have many dependencis. For example, you are developing an online store, where you have a cart. You can apply coupons to your cart, your cart can consist of many items, which refer to many products which refer to many prices or translations, which refer to... Now you want to send a cart to a client. Usually you don't want to send this infinite chain of dependencies, not even all of the fields of the cart. You have to first: delete fields that are not even convertable, like meta data fields, then delete all extra fields that are not needed by client, then do some processing on some fields. And your processing depends on mysterious 'params' determined in a runtime. Coding all of this for each case, for each entity, becomes such a pain. And even more pain it is for a person who is trying to figure out, what this code is doing. With JViewer you just need to explicitly delcare WHAT you want your data to look like after processing, and it will do the job for you. Example Let's go back to the cart case. After fetching needed entry with all possibly needed assocs preloaded, you have something like this: %{ ... coupon_id : nil , currency : %{ char : &quot;د.إ&quot; , code : &quot;AED&quot; , ... updated_at : ~N[2021-08-04 04:24:05] } , currency_id : 174 , id : 2 , inserted_at : ~N[2021-08-04 04:24:06] , items : [ %{ coupon_discount : 0 , currency_code : &quot;AED&quot; , id : 2 , inserted_at : ~N[2021-08-04 04:24:06] , part_of_item_id : nil , ... optional_products : [ ... ] , part_of_item_id : nil , product : %{ url : &quot;/product/correct slug&quot; , ... translations : [ %{ ... short_description : nil , title : &quot;Random product&quot; } , ... ] , title : &quot;&quot; , ... } , ... } ] , ... } To get a look at the full data sample, see test/j_viewer_test.exs. On a client we need coupon_id , id , also we need to add currency_code field with a value of a code of a currency. And we need some information about items dependency: array of items, where each item is an object with fields coupon_discount , inserted_at (for whatever reason), part_of_item_id , product . From our product we need only a title, translated to the language of a user, that sent the request. To achieve all of that with JViewer, we just say what we want! defmodule MyAppWeb.CartPresenter do import JViewer @return_schema object ( fields : [ field ( key : &quot;id&quot; , type : number ( ) ) , field ( key : &quot;currency_code&quot; , source_key : &quot;currency&quot; , # Your custom handler, # For more info see Handlers docs handler : MyAppWeb.CartView.CurrencyCodeHandler ) , field ( key : &quot;coupon_id&quot; , type : number ( ) , allow_null : true ) , field ( key : &quot;items&quot; , type : array ( type : object ( fields : [ field ( key : &quot;coupon_discount&quot; , type : number ( ) ) , field ( key : &quot;inserted_at&quot; , type : string ( ) ) , field ( key : &quot;part_of_item_id&quot; , type : number ( ) , allow_null : true ) , field ( key : &quot;product&quot; , type : object ( fields : [ field ( key : &quot;title&quot; , type : string ( ) , handler : JViewer.Handlers.Translator ) ] ) ) ] ) ) ) ] ) def view ( % MyApp.Cart { } = cart , %{ lang : lang } ) do return_schema = JViewer.Handler . put_params ( @return_schema , [ &quot;items&quot; , &quot;product&quot; , &quot;title&quot; ] , %{ key : :title , lang : lang } ) represent ( cart , return_schema ) end end If let's say our user needs English, here is result: %{ &quot;coupon_id&quot; =&gt; nil , &quot;currency_code&quot; =&gt; &quot;AED&quot; , &quot;id&quot; =&gt; 2 , &quot;items&quot; =&gt; [ %{ &quot;coupon_discount&quot; =&gt; 0 , &quot;inserted_at&quot; =&gt; &quot;2021-08-04 04:24:06&quot; , &quot;part_of_item_id&quot; =&gt; nil , &quot;product&quot; =&gt; %{ &quot;title&quot; =&gt; &quot;Random product&quot; } } ] }","ref":"JViewer.html","title":"JViewer","type":"module"},{"doc":"","ref":"JViewer.html#array/1","title":"JViewer.array/1","type":"function"},{"doc":"","ref":"JViewer.html#boolean/1","title":"JViewer.boolean/1","type":"function"},{"doc":"","ref":"JViewer.html#field/1","title":"JViewer.field/1","type":"function"},{"doc":"","ref":"JViewer.html#number/1","title":"JViewer.number/1","type":"function"},{"doc":"","ref":"JViewer.html#object/1","title":"JViewer.object/1","type":"function"},{"doc":"Represents data in a JSON encodable format according to return_schema . If data cannot be represented, an exception is thrown.","ref":"JViewer.html#represent/2","title":"JViewer.represent/2","type":"function"},{"doc":"","ref":"JViewer.html#string/1","title":"JViewer.string/1","type":"function"},{"doc":"JViewer.Handler is a way to take control over data processing in situations when data representation depends on runtime information, or simply when you want to process data in your own way. You can find handlers in JViewer.Handlers, or build a custom handler yourself! A handler should follow a given behaviuor and implement a function call , which takes super_data as a first argument and params as second argument. super_data A handler is always applyed to a field of an object, hence super_data is an object in your data.","ref":"JViewer.Handler.html","title":"JViewer.Handler","type":"behaviour"},{"doc":"","ref":"JViewer.Handler.html#c:call/2","title":"JViewer.Handler.call/2","type":"callback"},{"doc":"Puts handler in schema's field under path . Example @return_schema object ( fields : [ field ( key : &quot;product&quot; , type : object ( fields : [ ... ] ) ) ] ) As result of calling put_handler ( @return_schema , [ &quot;product&quot; ] , MyApp.ProductHandler ) We will have object ( fields : [ field ( key : &quot;product&quot; , type : object ( fields : [ ... ] ) , handler : MyApp.ProductHandler ) ] )","ref":"JViewer.Handler.html#put_handler/3","title":"JViewer.Handler.put_handler/3","type":"function"},{"doc":"Puts handler params in schema's field under _path . Example @return_schema object ( fields : [ field ( key : &quot;product&quot; , type : object ( fields : [ ... ] ) , handler : MyApp.ProductHandler ) ] ) As result of calling put_handler_params ( @return_schema , [ &quot;product&quot; ] , %{ key : &quot;value&quot; } ) We will have object ( fields : [ field ( key : &quot;product&quot; , type : object ( fields : [ ... ] ) , handler : MyApp.ProductHandler , handler_params : %{ key : &quot;value&quot; } ) ] )","ref":"JViewer.Handler.html#put_params/3","title":"JViewer.Handler.put_params/3","type":"function"},{"doc":"","ref":"JViewer.Handler.html#t:params/0","title":"JViewer.Handler.params/0","type":"type"},{"doc":"","ref":"JViewer.Handler.html#t:super_data/0","title":"JViewer.Handler.super_data/0","type":"type"},{"doc":"JViewer translations handler","ref":"JViewer.Handlers.Translator.html","title":"JViewer.Handlers.Translator","type":"module"},{"doc":"Translates value under key according to translations , where translations is a list of structs that look like %{ language : code # string like &quot;en&quot; key : translation # translation for a given key } Return an empty string if no translation is found .","ref":"JViewer.Handlers.Translator.html#call/2","title":"JViewer.Handlers.Translator.call/2","type":"function"},{"doc":"Defines a single callback apply_schema . First argument of apply_schema must be a struct (schema or part of schema) that implements given behaviour. Second argumet is data schema applying to.","ref":"JViewer.Types.html","title":"JViewer.Types","type":"behaviour"},{"doc":"","ref":"JViewer.Types.html#c:apply_schema/2","title":"JViewer.Types.apply_schema/2","type":"callback"},{"doc":"","ref":"JViewer.Types.Array.html","title":"JViewer.Types.Array","type":"module"},{"doc":"","ref":"JViewer.Types.Array.html#t:t/0","title":"JViewer.Types.Array.t/0","type":"type"},{"doc":"","ref":"JViewer.Types.Boolean.html","title":"JViewer.Types.Boolean","type":"module"},{"doc":"","ref":"JViewer.Types.Boolean.html#t:t/0","title":"JViewer.Types.Boolean.t/0","type":"type"},{"doc":"","ref":"JViewer.Types.Number.html","title":"JViewer.Types.Number","type":"module"},{"doc":"","ref":"JViewer.Types.Number.html#t:t/0","title":"JViewer.Types.Number.t/0","type":"type"},{"doc":"","ref":"JViewer.Types.Object.html","title":"JViewer.Types.Object","type":"module"},{"doc":"","ref":"JViewer.Types.Object.html#t:t/0","title":"JViewer.Types.Object.t/0","type":"type"},{"doc":"","ref":"JViewer.Types.Object.Field.html","title":"JViewer.Types.Object.Field","type":"module"},{"doc":"","ref":"JViewer.Types.Object.Field.html#t:t/0","title":"JViewer.Types.Object.Field.t/0","type":"type"},{"doc":"","ref":"JViewer.Types.String.html","title":"JViewer.Types.String","type":"module"},{"doc":"","ref":"JViewer.Types.String.html#t:t/0","title":"JViewer.Types.String.t/0","type":"type"},{"doc":"","ref":"JViewer.FieldException.html","title":"JViewer.FieldException","type":"exception"},{"doc":"","ref":"JViewer.FieldException.html#add_to_path/2","title":"JViewer.FieldException.add_to_path/2","type":"function"}]